@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2013-2016 Vinnie Falco (vinnie dot falco at gmail dot com)
+// Copyright (c) 2013-2017 Vinnie Falco (vinnie dot falco at gmail dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -8,12 +8,13 @@
 #ifndef BEAST_WEBSOCKET_ERROR_HPP
 #define BEAST_WEBSOCKET_ERROR_HPP
 
+#include <beast/config.hpp>
 #include <beast/core/error.hpp>
 
 namespace beast {
 namespace websocket {
 
-/// Error codes returned from @ref stream operations.
+/// Error codes returned from @ref beast::websocket::stream operations.
 enum class error
 {
     /// Both sides performed a WebSocket close
@@ -50,11 +51,6 @@ enum class error
     general
 };
 
-#if ! GENERATING_DOCS
-error_code
-make_error_code(error e);
-#endif
-
 } // websocket
 } // beast
 
