@@ -1,5 +1,5 @@
 //
-// Copyright (c) 2013-2016 Vinnie Falco (vinnie dot falco at gmail dot com)
+// Copyright (c) 2013-2017 Vinnie Falco (vinnie dot falco at gmail dot com)
 //
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
@@ -19,7 +19,7 @@ public:
     void run() override
     {
         for(int i = 1; i <= 999; ++i)
-            expect(reason_string(i) != nullptr);
+            BEAST_EXPECT(reason_string(i) != nullptr);
     }
 };
 
