@@ -1,4 +1,3 @@
-#include "acconfig.h"
 #include "arch/probe.h"
 
 /* flags we export */
@@ -19,13 +18,10 @@ int ceph_arch_aarch64_crc32 = 0;
 static unsigned long get_auxval(unsigned long type)
 {
 	unsigned long result = 0;
-	int read = 0;
 	FILE *f = fopen("/proc/self/auxv", "r");
 	if (f) {
 		ElfW(auxv_t) entry;
-		while ((read = fread(&entry, sizeof(entry), 1, f)) > 0) {
-			if (read != 1)
-				break;
+		while (fread(&entry, sizeof(entry), 1, f) == 1) {
 			if (entry.a_type == type) {
 				result = entry.a_un.a_val;
 				break;
@@ -49,8 +45,10 @@ int ceph_arch_arm_probe(void)
 	ceph_arch_neon = (get_hwcap() & HWCAP_NEON) == HWCAP_NEON;
 #elif __aarch64__ && __linux__
 	ceph_arch_neon = (get_hwcap() & HWCAP_ASIMD) == HWCAP_ASIMD;
-# if defined(HAVE_ARMV8_CRC) && defined(HWCAP_CRC32)
+# ifdef HWCAP_CRC32
 	ceph_arch_aarch64_crc32 = (get_hwcap() & HWCAP_CRC32) == HWCAP_CRC32;
+# else
+	ceph_arch_aarch64_crc32 = 0;  // sorry!
 # endif
 #else
 	if (0)
